import { useCallback, useState } from "react";
import {
  dispatch,
  useCurrentRound,
  useTargetPoint,
} from "../../../../domain/gameStore";
import ClickMarkerMap from "./ClickMarkerMap";
import styles from "./GuessPane.module.css";
import RoundTimer from "./RoundTimer";
import GunGame from "./GunGame";
import { useMapResizeKeybindings } from "./hooks";
import { useGameConfig } from "../../../../hooks/useGameInfo";
import { GUN_GAME } from "../../../../domain/constants";
import { checkScore } from "../../../../domain/apiMethods";

const mapSizeOpts = {
  small: styles["pane--small"],
  medium: styles["pane--medium"],
  big: styles["pane--big"],
};

const toggleMapSize = larger => b =>
  b === "big" || b === "medium" ? "small" : larger;

const GuessPane = () => {
  const [selectedPoint, setSelectedPoint] = useState(null);
  const [submitted, setSubmitted] = useState(false);
  const [mapSize, setMapSize] = useState("small");
  const [potentialScore, setPotentialScore] = useState(null);
  const [gunGameBlock, setGunGameBlock] = useState(false);
  const unblock = useCallback(() => setGunGameBlock(false), []);
  const toggleBig = useCallback(() => setMapSize(toggleMapSize("big")), []);
  const toggleMed = useCallback(() => setMapSize(toggleMapSize("medium")), []);
  useMapResizeKeybindings(toggleBig);
  const { gameMode, scoreMethod } = useGameConfig();
  const targetPoint = useTargetPoint();
  const roundNum = useCurrentRound();

  const handleSubmitGuess = async () => {
    if (gameMode === GUN_GAME) {
      const { score } = await checkScore(
        selectedPoint,
        targetPoint,
        scoreMethod,
        roundNum
      );
      if (score < 4000) {
        setPotentialScore(score);
        setGunGameBlock(true);
        return;
      }
    }
    setSubmitted(true);
    if (!submitted) {
      await dispatch.submitGuess(selectedPoint);
    }
  };

  return (
    <>
      {gunGameBlock && gameMode === GUN_GAME && (
        <GunGame points={potentialScore} onFinish={unblock} />
      )}
      <div className={`${styles.pane} ${mapSizeOpts[mapSize]}`}>
        <button
          type="button"
          className={styles.submit}
          onClick={handleSubmitGuess}
          disabled={submitted || selectedPoint === null}
        >
          Submit Guess
        </button>
        <ClickMarkerMap onMarkerMoved={setSelectedPoint} />
        <RoundTimer onTimeout={handleSubmitGuess} />
        <div
          className={styles.resize}
          onClick={toggleBig}
          role="button"
          tabIndex="0"
          onKeyDown={({ key }) => {
            if (key === "Enter") {
              toggleBig();
            }
          }}
        >
          {mapSize === "small" ? "↗️" : "↙️"}
        </div>
        <div
          className={`${styles.resize} ${styles["resize--medium"]}`}
          onClick={toggleMed}
          role="button"
          tabIndex="0"
          onKeyDown={({ key }) => {
            if (key === "Enter") {
              toggleMed();
            }
          }}
        >
          {mapSize === "small" ? "➡️" : "⬅️"}
        </div>
      </div>
    </>
  );
};

export default GuessPane;
