import iso from "iso-3166-1";
import flagLookup from "../../../../domain/flagLookup";
import styles from "./ScoreBoard.module.css";

// since everyone wants to use giant names...
const bigNameMin = 30;
const medNameMin = 20;

const getNameClass = name => {
  if (name.length >= bigNameMin) {
    return styles.longname;
  }
  if (name.length >= medNameMin) {
    return styles.medname;
  }
  return "";
};

export const PlayerScoreTile = ({ name, guesses, totalScore, winner }) => (
  <div className={styles.tile}>
    <div className={styles.header}>
      <span className={`${styles.name} ${getNameClass(name)}`}>{name}</span>
      <span className={styles.total}>
        {winner && "🏆"} {totalScore}
      </span>
    </div>
    <div className={styles.scores}>
      {Object.entries(guesses)
        .map(([k, v]) => [parseInt(k, 10), v])
        .sort(([k1], [k2]) => (k1 < k2 ? -1 : 1))
        .map(([num, { score, country }]) =>
          score !== undefined ? (
            <div className={styles.round} key={num}>
              <span className={styles.score}>
                Round {num}: {score ?? 0}
              </span>
              <span
                className={styles.flag}
                title={country && iso.whereAlpha2(country)?.country}
              >
                {flagLookup(country)}
              </span>
            </div>
          ) : null
        )}
    </div>
  </div>
);

const ScoreBoard = ({ players, requireFinished }) => (
  <div className={styles.scoreboard}>
    {(requireFinished
      ? players.filter(({ currentRound }) => currentRound === null)
      : players.slice()
    )
      .sort(({ totalScore: p1 }, { totalScore: p2 }) =>
        // eslint-disable-next-line no-nested-ternary
        p1 > p2 ? -1 : p1 < p2 ? 1 : 0
      )
      .map(({ name, guesses, totalScore }, i) => (
        <PlayerScoreTile
          key={name}
          // TODO gracefully handle ties...
          winner={i === 0}
          {...{ name, guesses, totalScore }}
        />
      ))}
  </div>
);

export default ScoreBoard;
